# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for Database Migration Service commands."""


from googlecloudsdk.api_lib.storage import storage_util
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def ConnectionProfileAttributeConfig(name='connection_profile'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The connection profile of the {resource}.')


def MigrationJobAttributeConfig(name='migration_job'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The migration job of the {resource}.')


def PrivateConnectionAttributeConfig(name='private_connection'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The private connection of the {resource}.')


def ServiceAttachmentAttributeConfig(name='service_attachment'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The service attachment of the {resource}.'
  )


def ConversionWorkspaceAttributeConfig(name='conversion_workspace'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The conversion workspace of the {resource}.'
  )


def CmekKeyAttributeConfig(name='cmek-key'):
  # For anchor attribute, help text is generated automatically.
  return concepts.ResourceParameterAttributeConfig(name=name)


def CmekKeyringAttributeConfig(name='cmek-keyring'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The CMEK keyring id of the {resource}.'
  )


def CmekProjectAttributeConfig(name='cmek-project'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The Cloud project id for the {resource}.'
  )


def RegionAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='region', help_text='The Cloud region for the {resource}.')


def GetRegionResourceSpec(resource_name='region'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations',
      resource_name=resource_name,
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetConnectionProfileResourceSpec(resource_name='connection_profile'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations.connectionProfiles',
      resource_name=resource_name,
      connectionProfilesId=ConnectionProfileAttributeConfig(name=resource_name),
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetMigrationJobResourceSpec(resource_name='migration_job'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations.migrationJobs',
      resource_name=resource_name,
      migrationJobsId=MigrationJobAttributeConfig(name=resource_name),
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False,
  )


def GetPrivateConnectionResourceSpec(resource_name='private_connection'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations.privateConnections',
      resource_name=resource_name,
      api_version='v1',
      privateConnectionsId=PrivateConnectionAttributeConfig(name=resource_name),
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetServiceAttachmentResourceSpec(resource_name='service_attachment'):
  return concepts.ResourceSpec(
      'compute.serviceAttachments',
      resource_name=resource_name,
      serviceAttachment=ServiceAttachmentAttributeConfig(name=resource_name),
      project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def GetConversionWorkspaceResourceSpec(resource_name='conversion_workspace'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations.conversionWorkspaces',
      resource_name=resource_name,
      api_version='v1',
      conversionWorkspacesId=ConversionWorkspaceAttributeConfig(
          name=resource_name),
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetCmekKeyResourceSpec(resource_name='cmek-key'):
  return concepts.ResourceSpec(
      'cloudkms.projects.locations.keyRings.cryptoKeys',
      resource_name=resource_name,
      api_version='v1',
      cryptoKeysId=CmekKeyAttributeConfig(),
      keyRingsId=CmekKeyringAttributeConfig(),
      locationsId=RegionAttributeConfig(),
      projectsId=CmekProjectAttributeConfig(),
      disable_auto_completers=False,
  )


def GetKMSKeyResourceSpec(resource_name='kms-key'):
  return concepts.ResourceSpec(
      'cloudkms.projects.locations.keyRings.cryptoKeys',
      resource_name=resource_name,
      api_version='v1',
      cryptoKeysId=CmekKeyAttributeConfig('kms-key'),
      keyRingsId=CmekKeyringAttributeConfig('kms-keyring'),
      locationsId=RegionAttributeConfig(),
      projectsId=CmekProjectAttributeConfig('kms-project'),
      disable_auto_completers=False,
  )


def AddConnectionProfileResourceArg(parser, verb, positional=True):
  """Add a resource argument for a database migration connection profile.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'connection_profile'
  else:
    name = '--connection-profile'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetConnectionProfileResourceSpec(),
      'The connection profile {}.'.format(verb),
      required=True).AddToParser(parser)


def AddCloudSqlConnectionProfileResourceArgs(parser, verb):
  """Add resource arguments for a database migration CloudSQL connection profile.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
  """
  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          'connection_profile',
          GetConnectionProfileResourceSpec(),
          'The connection profile {}.'.format(verb),
          required=True),
      presentation_specs.ResourcePresentationSpec(
          '--source-id',
          GetConnectionProfileResourceSpec(),
          'Database Migration Service source connection profile ID.',
          required=True,
          flag_name_overrides={'region': ''}),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--source-id.region': ['--region']
      }).AddToParser(parser)


def AddAlloyDBConnectionProfileResourceArgs(parser, verb):
  """Add resource arguments for a database migration AlloyDB connection profile.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
  """
  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          'connection_profile',
          GetConnectionProfileResourceSpec(),
          'The connection profile {}.'.format(verb),
          required=True,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--kms-key',
          GetKMSKeyResourceSpec(),
          (
              'Name of the CMEK (customer-managed encryption key) used for this'
              ' AlloyDB cluster. For example,'
              ' projects/myProject/locations/us-central1/keyRings/myKeyRing/cryptoKeys/myKey.'
          ),
          flag_name_overrides={'region': ''},
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={'--kms-key.region': ['--region']},
  ).AddToParser(parser)


def AddCmekResourceArgs(parser):
  """Add a resource argument for a connection profile cmek.

  Args:
    parser: the parser for the command.
  """
  concept_parsers.ConceptParser.ForResource(
      '--cmek-key',
      GetCmekKeyResourceSpec(),
      'Name of the CMEK (customer-managed encryption key) used for'
      ' the connection profile. For example,'
      ' projects/myProject/locations/us-central1/keyRings/myKeyRing/cryptoKeys/myKey.',
      flag_name_overrides={'region': ''},
  ).AddToParser(parser)


def AddOracleConnectionProfileResourceArg(parser,
                                          verb,
                                          positional=True,
                                          required=True):
  """Add a resource argument for a database migration oracle cp.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
    required: bool, if True, means that a flag is required.
  """
  if positional:
    name = 'connection_profile'
  else:
    name = '--connection-profile'

  connectivity_parser = parser.add_group(mutex=True)
  connectivity_parser.add_argument(
      '--static-ip-connectivity',
      action='store_true',
      help="""use static ip connectivity""")

  forward_ssh_parser = connectivity_parser.add_group()
  forward_ssh_parser.add_argument(
      '--forward-ssh-hostname',
      help="""Hostname for the SSH tunnel.""",
      required=required)
  forward_ssh_parser.add_argument(
      '--forward-ssh-username',
      help="""Username for the SSH tunnel.""",
      required=required)
  forward_ssh_parser.add_argument(
      '--forward-ssh-port',
      help="""Port for the SSH tunnel, default value is 22.\
      """,
      default=22)
  password_group = forward_ssh_parser.add_group(required=required, mutex=True)
  password_group.add_argument(
      '--forward-ssh-password', help="""\
          SSH password.
          """)
  password_group.add_argument(
      '--forward-ssh-private-key', help='SSH private key..')

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetConnectionProfileResourceSpec(),
          'The connection profile {}.'.format(verb),
          required=True),
      presentation_specs.ResourcePresentationSpec(
          '--private-connection',
          GetPrivateConnectionResourceSpec(),
          'Resource ID of the private connection.',
          flag_name_overrides={'region': ''},
          group=connectivity_parser)
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--private-connection.region': ['--region']
      }).AddToParser(parser)


def AddSqlServerConnectionProfileResourceArg(parser,
                                             verb,
                                             positional=True):
  """Add a resource argument for a database migration sqlserver cp.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'connection_profile'
  else:
    name = '--connection-profile'

  connectivity_parser = parser.add_group(mutex=True)
  connectivity_parser.add_argument(
      '--static-ip-connectivity',
      action='store_true',
      help="""use static ip connectivity""")

  AddForwardSshConnectivityGroup(connectivity_parser)

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetConnectionProfileResourceSpec(),
          'The connection profile {}.'.format(verb),
          required=True),
      presentation_specs.ResourcePresentationSpec(
          '--private-connection',
          GetPrivateConnectionResourceSpec(),
          'Resource ID of the private connection.',
          flag_name_overrides={'region': ''},
          group=connectivity_parser),
      presentation_specs.ResourcePresentationSpec(
          '--psc-service-attachment',
          GetServiceAttachmentResourceSpec(),
          'Resource ID of the service attachment.',
          flag_name_overrides={'region': ''},
          group=connectivity_parser,
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--private-connection.region': ['--region'],
          '--psc-service-attachment.region': ['--region'],
      }).AddToParser(parser)


def AddPostgresqlConnectionProfileResourceArg(parser, verb, positional=True):
  """Add a resource argument for a database migration postgresql cp.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'connection_profile'
  else:
    name = '--connection-profile'

  connectivity_parser = parser.add_group(mutex=True)
  connectivity_parser.add_argument(
      '--static-ip-connectivity',
      action='store_true',
      help="""use static ip connectivity""",
  )

  AddForwardSshConnectivityGroup(connectivity_parser)

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetConnectionProfileResourceSpec(),
          'The connection profile {}.'.format(verb),
          required=True,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--psc-service-attachment',
          GetServiceAttachmentResourceSpec(),
          'Resource ID of the service attachment.',
          flag_name_overrides={'region': ''},
          group=connectivity_parser,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--private-connection',
          GetPrivateConnectionResourceSpec(),
          'Resource ID of the private connection.',
          flag_name_overrides={'region': ''},
          group=connectivity_parser,
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--psc-service-attachment.region': ['--region'],
          '--private-connection.region': ['--region'],
      },
  ).AddToParser(parser)


def AddForwardSshConnectivityGroup(parser, hidden=False):
  """Adds a group for forward ssh connectivity.

  Args:
    parser: the parser for the command.
    hidden: whether the group is hidden.
  """
  forward_ssh_parser = parser.add_group(hidden=hidden)
  forward_ssh_parser.add_argument(
      '--forward-ssh-hostname',
      help="""Hostname for the SSH tunnel.""",
      required=True)
  forward_ssh_parser.add_argument(
      '--forward-ssh-username',
      help="""Username for the SSH tunnel.""",
      required=True)
  forward_ssh_parser.add_argument(
      '--forward-ssh-port',
      help="""Port for the SSH tunnel, default value is 22.\
      """,
      default=22)
  password_group = forward_ssh_parser.add_group(required=True, mutex=True)
  password_group.add_argument(
      '--forward-ssh-password', help="""\
          SSH password.
          """)
  password_group.add_argument(
      '--forward-ssh-private-key', help='SSH private key..')


def AddOnlyMigrationJobResourceArgs(parser, verb, positional=True):
  """Add resource arguments for actions on an mj except create/update.

  Args:
    parser: argparse.ArgumentParser, the parser for the command.
    verb: str, the verb to describe the resource, such as 'to promote'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'migration_job'
  else:
    name = '--migration-job'

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetMigrationJobResourceSpec(),
          'The migration job {}.'.format(verb),
          required=True,
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
  ).AddToParser(parser)


def AddMigrationJobResourceArgs(parser, verb, required=False):
  """Add resource arguments for creating/updating a database migration job.

  Args:
    parser: argparse.ArgumentParser, the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    required: boolean, whether source/dest resource args are required.
  """
  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          'migration_job',
          GetMigrationJobResourceSpec(),
          'The migration job {}.'.format(verb),
          required=True,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--source',
          GetConnectionProfileResourceSpec(),
          (
              'ID of the source connection profile, representing the source'
              ' database.'
          ),
          required=required,
          flag_name_overrides={'region': ''},
      ),
      presentation_specs.ResourcePresentationSpec(
          '--destination',
          GetConnectionProfileResourceSpec(),
          (
              'ID of the destination connection profile, representing the '
              'destination database.'
          ),
          required=required,
          flag_name_overrides={'region': ''},
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--source.region': ['--region'],
          '--destination.region': ['--region'],
      },
  ).AddToParser(parser)


def AddHeterogeneousMigrationJobResourceArgs(parser, verb, required=False):
  """Add resource arguments for creating/updating a heterogeneous database mj.

  Args:
    parser: argparse.ArgumentParser, the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    required: boolean, whether source/dest resource args are required.
  """
  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          'migration_job',
          GetMigrationJobResourceSpec(),
          'The migration job {}.'.format(verb),
          required=True,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--source',
          GetConnectionProfileResourceSpec(),
          (
              'ID of the source connection profile, representing the source'
              ' database.'
          ),
          required=required,
          flag_name_overrides={'region': ''},
      ),
      presentation_specs.ResourcePresentationSpec(
          '--destination',
          GetConnectionProfileResourceSpec(),
          (
              'ID of the destination connection profile, representing the '
              'destination database.'
          ),
          required=required,
          flag_name_overrides={'region': ''},
      ),
      presentation_specs.ResourcePresentationSpec(
          '--conversion-workspace',
          GetConversionWorkspaceResourceSpec(),
          'Name of the conversion workspaces to be used for the migration job',
          flag_name_overrides={'region': ''},
      ),
      presentation_specs.ResourcePresentationSpec(
          '--original-migration-name',
          GetMigrationJobResourceSpec(),
          'Name of the original migration job to be used for the failback'
          ' migration job',
          flag_name_overrides={'region': ''},
      ),
      presentation_specs.ResourcePresentationSpec(
          '--cmek-key',
          GetCmekKeyResourceSpec(),
          (
              'Name of the CMEK (customer-managed encryption key) used for'
              ' the migration job'
          ),
          flag_name_overrides={'region': ''},
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--source.region': ['--region'],
          '--destination.region': ['--region'],
          '--conversion-workspace.region': ['--region'],
          '--original-migration-name.region': ['--region'],
          '--cmek-key.region': ['--region'],
      },
  ).AddToParser(parser)


def GetVpcResourceSpec():
  """Constructs and returns the Resource specification for VPC."""

  def VpcAttributeConfig():
    return concepts.ResourceParameterAttributeConfig(
        name='vpc',
        help_text=(
            'fully qualified name of the VPC database migration will peer to.'
        ),
    )

  return concepts.ResourceSpec(
      'compute.networks',
      resource_name='vpc',
      network=VpcAttributeConfig(),
      project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def AddPrivateConnectionResourceArg(parser, verb, positional=True):
  """Add a resource argument for a database migration private connection.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'private_connection'
  else:
    name = '--private-connection'

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetPrivateConnectionResourceSpec(),
          'The private connection {}.'.format(verb),
          required=True,
      ),
  ]
  concept_parsers.ConceptParser(resource_specs).AddToParser(parser)


def AddPrivateConnectionDeleteResourceArg(parser, verb, positional=True):
  """Add a resource argument for a database migration private connection.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'private_connection'
  else:
    name = '--private-connection'

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetPrivateConnectionResourceSpec(),
          'The private connection {}.'.format(verb),
          required=True,
      )
  ]
  concept_parsers.ConceptParser(resource_specs).AddToParser(parser)


def AddConversionWorkspaceResourceArg(parser, verb, positional=True):
  """Add a resource argument for a database migration conversion workspace.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'conversion_workspace'
  else:
    name = '--conversion-workspace'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetConversionWorkspaceResourceSpec(),
      'The conversion workspace {}.'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddConversionWorkspaceSeedResourceArg(parser, verb, positional=True):
  """Add a resource argument for seeding a database migration cw.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to seed'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'conversion_workspace'
  else:
    name = '--conversion-workspace'

  # Top-level mutually exclusive group for seed source
  seed_from_group = parser.add_group(
      mutex=True,
      required=True,
      help='The source for seeding the conversion workspace.',
  )

  seed_from_group.add_argument(
      '--gcs-path',
      type=storage_util.ObjectReference.FromArgument,
      help=(
          'The Cloud Storage path containing the schema report files. '
          'Must be in the format gs://bucket/prefix.'
      ),
  )

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetConversionWorkspaceResourceSpec(),
          'The conversion workspace {}.'.format(verb),
          required=True,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--source-connection-profile',
          GetConnectionProfileResourceSpec(),
          'The connection profile {} from.'.format(verb),
          flag_name_overrides={'region': ''},
          group=seed_from_group,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--destination-connection-profile',
          GetConnectionProfileResourceSpec(),
          'The connection profile {} from.'.format(verb),
          flag_name_overrides={'region': ''},
          group=seed_from_group,
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--source-connection-profile.region': ['--region'],
          '--destination-connection-profile.region': ['--region'],
      }).AddToParser(parser)


def AddConversionWorkspaceApplyResourceArg(parser, verb, positional=True):
  """Add a resource argument for applying a database migration cw.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to apply'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'conversion_workspace'
  else:
    name = '--conversion-workspace'

  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          name,
          GetConversionWorkspaceResourceSpec(),
          'The conversion workspace {}.'.format(verb),
          required=True,
      ),
      presentation_specs.ResourcePresentationSpec(
          '--destination-connection-profile',
          GetConnectionProfileResourceSpec(),
          'The connection profile {} to.'.format(verb),
          flag_name_overrides={'region': ''},
          required=True,
      ),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--destination-connection-profile.region': ['--region']
      }).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility for updating Looker instances.

This utility is primarily used for modifying request hooks for update
requests for Looker instances. See go/gcloud-creating-commands#request-hooks
for more details.
"""

from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core.console import console_io

_MSG_RELEASE_CHANNEL_UPGRADE_WARNING = (
    'Caution: Your instance will be updated to the latest version available in'
    ' the {selected_channel} channel. For more details, review'
    ' https://cloud.google.com/looker/docs/looker-core-release-process#release_channels.'
)

_MSG_RELEASE_CHANNEL_DOWNGRADE_WARNING = (
    'Caution: Your instance will remain on version {looker_version} until this'
    ' version becomes available in the {selected_channel} channel or may update'
    ' instantly if the {selected_channel} channel is on a higher version. For'
    ' more details, review'
    ' https://cloud.google.com/looker/docs/looker-core-release-process#release_channels.'
)

_MSG_RELEASE_CHANNEL_RAPID_MAINTENANCE_WARNING = (
    'Switching to the Rapid channel automatically removes the maintenance'
    ' window and deny maintenance periods for this instance.'
)


def _GetReleaseChannelChangeType(*, current, selected, release_channel_enum):
  """Checks if release channel change is an upgrade or downgrade.

  Args:
    current: The current release channel of the instance.
    selected: The new release channel selected by the user.
    release_channel_enum: The release channel enum from the API messages.

  Returns:
    'upgrade', 'downgrade', or None if the rank is the same.
  """
  current_val = (
      current
      if current is not None
      else release_channel_enum.RELEASE_CHANNEL_UNSPECIFIED
  )

  rank_by_release_channel = {
      release_channel_enum.RELEASE_CHANNEL_UNSPECIFIED: 0.5,
      release_channel_enum.RAPID: 0,
      release_channel_enum.REGULAR: 1,
      release_channel_enum.STABLE: 2,
  }

  current_rank = rank_by_release_channel.get(current_val, 0)
  selected_rank = rank_by_release_channel.get(selected, 0)

  if current_rank == selected_rank:
    return None
  return 'upgrade' if selected_rank < current_rank else 'downgrade'


def _GetReleaseChannelWarningMessage(
    *, current, selected, looker_version, release_channel_enum
):
  """Gets warning message for release channel change.

  Args:
    current: The current release channel of the instance.
    selected: The new release channel selected by the user.
    looker_version: The current version of the Looker instance.
    release_channel_enum: The release channel enum from the API messages.

  Returns:
    A warning message string or None if no warning is needed.
  """
  change_type = _GetReleaseChannelChangeType(
      current=current,
      selected=selected,
      release_channel_enum=release_channel_enum,
  )
  if change_type is None:
    return None

  label_by_release_channel = {
      release_channel_enum.RELEASE_CHANNEL_UNSPECIFIED: 'No Channel',
      release_channel_enum.RAPID: 'Rapid',
      release_channel_enum.REGULAR: 'Regular',
      release_channel_enum.STABLE: 'Stable',
  }

  selected_label = label_by_release_channel.get(selected, str(selected))

  if change_type == 'downgrade':
    warning_msg = _MSG_RELEASE_CHANNEL_DOWNGRADE_WARNING.format(
        looker_version=looker_version, selected_channel=selected_label
    )
  else:
    warning_msg = _MSG_RELEASE_CHANNEL_UPGRADE_WARNING.format(
        selected_channel=selected_label
    )

  if (
      selected == release_channel_enum.RAPID
      and current != release_channel_enum.RAPID
  ):
    warning_msg += '\n\n' + _MSG_RELEASE_CHANNEL_RAPID_MAINTENANCE_WARNING

  return warning_msg


def _WarnForAdminSettingsUpdate():
  """Adds prompt that warns about allowed email domains update."""
  message = 'Change to instance allowed email domain requested. '
  message += (
      'Updating the allowed email domains from cli means the value provided'
      ' will be considered as the entire list and not an amendment to the'
      ' existing list of allowed email domains.'
  )
  console_io.PromptContinue(
      message=message,
      prompt_string='Do you want to proceed with update?',
      cancel_on_no=True,
  )


def _WarnForPscAllowedVpcsUpdate():
  """Adds prompt that warns about allowed vpcs update."""
  message = (
      'Change to instance PSC allowed Virtual Private Cloud networks'
      ' requested. '
  )
  message += (
      'Updating the allowed VPC networks from cli means the value provided'
      ' will be considered as the entire list and not an amendment to the'
      ' existing list of allowed vpcs.'
  )
  console_io.PromptContinue(
      message=message,
      prompt_string='Do you want to proceed with update?',
      cancel_on_no=True,
  )


def _WarnForPscAllowedVpcsRemovalUpdate():
  """Adds prompt that warns about allowed vpcs removal."""
  message = 'Removal of instance PSC allowed vpcs requested. '

  console_io.PromptContinue(
      message=message,
      prompt_string='Do you want to proceed with removal of PSC allowed vpcs?',
      cancel_on_no=True,
  )


def _WarnForPscServiceAttachmentsUpdate():
  """Adds prompt that warns about service attachments update."""
  message = 'Change to instance PSC service attachments requested. '
  message += (
      'Updating the PSC service attachments from cli means the value provided'
      ' will be considered as the entire list and not an amendment to the'
      ' existing list of PSC service attachments'
  )
  console_io.PromptContinue(
      message=message,
      prompt_string='Do you want to proceed with update?',
      cancel_on_no=True,
  )


def _WarnForPscServiceAttachmentsRemovalUpdate():
  """Adds prompt that warns about service attachments removal."""
  message = 'Removal of instance PSC service attachments requested. '

  console_io.PromptContinue(
      message=message,
      prompt_string=(
          'Do you want to proceed with removal of service attachments?'
      ),
      cancel_on_no=True,
  )


def AddFieldToUpdateMask(field, patch_request):
  """Adds fields to the update mask of the patch request.

  Args:
    field: the field of the update mask to patch request for Looker instances.
    patch_request: the request of the actual update command to be modified

  Returns:
    A patch request object to be sent to the server. The object is an instance
    of UpdateInstanceRequest: http://shortn/_yn9MhWaGJx
  """
  update_mask = patch_request.updateMask
  if update_mask:
    fields = set(update_mask.split(','))
    if field not in fields:
      patch_request.updateMask = f'{update_mask},{field}'
  else:
    patch_request.updateMask = field
  return patch_request


def ModifyAllowedEmailDomains(unused_instance_ref, args, patch_request):
  """Python hook to modify allowed email domains in looker instance update request."""
  if args.IsSpecified('allowed_email_domains'):
    # Changing allowed email domains means this list will be overwritten in the
    # DB and not amended and users should be warned before proceeding.
    _WarnForAdminSettingsUpdate()
    patch_request.instance.adminSettings.allowedEmailDomains = (
        args.allowed_email_domains
    )
    patch_request = AddFieldToUpdateMask(
        'admin_settings.allowed_email_domains', patch_request
    )
  return patch_request


def UpdateMaintenanceWindow(unused_instance_ref, args, patch_request):
  """Hook to update maintenance window to the update mask of the request."""
  if args.IsSpecified('maintenance_window_day') or args.IsSpecified(
      'maintenance_window_time'
  ):
    patch_request = AddFieldToUpdateMask('maintenance_window', patch_request)
  return patch_request


def UpdateEnablePublicIpAlpha(unused_instance_ref, args, patch_request):
  """Hook to update public IP to the update mask of the request for alpha."""
  if args.IsSpecified('enable_public_ip'):
    patch_request = AddFieldToUpdateMask('enable_public_ip', patch_request)
  return patch_request


def UpdatePublicIPEnabled(unused_instance_ref, args, patch_request):
  """Hook to update public IP to the update mask of the request fo GA."""
  if args.IsSpecified('public_ip_enabled'):
    patch_request = AddFieldToUpdateMask('public_ip_enabled', patch_request)
  return patch_request


def UpdateOauthClient(unused_instance_ref, args, patch_request):
  """Hook to update Oauth configs to the update mask of the request."""
  if args.IsSpecified('oauth_client_id') and args.IsSpecified(
      'oauth_client_secret'
  ):
    patch_request = AddFieldToUpdateMask(
        'oauth_config.client_id', patch_request
    )
    patch_request = AddFieldToUpdateMask(
        'oauth_config.client_secret', patch_request
    )
    # Disable shared OAuth when updating custom credentials.
    patch_request.instance.oauthConfig.sharedOauthClientEnabled = False
    patch_request = AddFieldToUpdateMask(
        'oauth_config.shared_oauth_client_enabled', patch_request
    )

  return patch_request


def UpdateDenyMaintenancePeriod(unused_instance_ref, args, patch_request):
  """Hook to update deny maintenance period to the update mask of the request."""
  if (
      args.IsSpecified('deny_maintenance_period_start_date')
      or args.IsSpecified('deny_maintenance_period_end_date')
      or args.IsSpecified('deny_maintenance_period_time')
  ):
    patch_request = AddFieldToUpdateMask(
        'deny_maintenance_period', patch_request
    )
  return patch_request


def UpdateUserMetadata(unused_instance_ref, args, patch_request):
  """Hook to update deny user metadata to the update mask of the request."""
  if (
      args.IsSpecified('add_viewer_users')
      or args.IsSpecified('add_standard_users')
      or args.IsSpecified('add_developer_users')
  ):
    patch_request = AddFieldToUpdateMask('user_metadata', patch_request)
  return patch_request


def UpdateCustomDomain(unused_instance_ref, args, patch_request):
  """Hook to update custom domain to the update mask of the request."""
  if args.IsSpecified('custom_domain'):
    patch_request = AddFieldToUpdateMask('custom_domain', patch_request)
  return patch_request


def UpdatePscAllowedVpcs(unused_instance_ref, args, patch_request):
  """Hook to update psc confing allowed vpcs to the update mask of the request."""
  if args.IsSpecified('psc_allowed_vpcs'):
    # Changing allowed email domains means this list will be overwritten in the
    # DB and not amended and users should be warned before proceeding.
    _WarnForPscAllowedVpcsUpdate()
    patch_request.instance.pscConfig.allowedVpcs = args.psc_allowed_vpcs
    patch_request = AddFieldToUpdateMask(
        'psc_config.allowed_vpcs', patch_request
    )
  elif args.IsSpecified('clear_psc_allowed_vpcs'):
    _WarnForPscAllowedVpcsRemovalUpdate()
    patch_request = AddFieldToUpdateMask(
        'psc_config.allowed_vpcs', patch_request
    )
  return patch_request


def UpdatePscServiceAttachments(unused_instance_ref, args, patch_request):
  """Hook to update psc confing service attachments to the update mask of the request."""
  if args.IsSpecified('psc_service_attachment'):
    _WarnForPscServiceAttachmentsUpdate()
    patch_request = AddFieldToUpdateMask(
        'psc_config.service_attachments', patch_request
    )
  elif args.IsSpecified('clear_psc_service_attachments'):
    _WarnForPscServiceAttachmentsRemovalUpdate()
    patch_request = AddFieldToUpdateMask(
        'psc_config.service_attachments', patch_request
    )
  return patch_request


def UpdateGeminiAiConfig(unused_instance_ref, args, patch_request):
  """Hook to update gemini enabled to the update mask of the request."""
  if args.IsSpecified('gemini_enabled'):
    patch_request = AddFieldToUpdateMask('gemini_enabled', patch_request)
  if args.IsSpecified('gemini_preview_tester_enabled'):
    patch_request = AddFieldToUpdateMask(
        'gemini_ai_config.trusted_tester', patch_request
    )
  if args.IsSpecified('gemini_prompt_log_enabled'):
    patch_request = AddFieldToUpdateMask(
        'gemini_ai_config.prompt_logging', patch_request
    )
  return patch_request


def UpdatePeriodicExportConfig(unused_instance_ref, args, patch_request):
  """Hook to handle periodic export config updates."""
  if (
      args.IsSpecified('clear_periodic_export_config')
      or args.IsSpecified('periodic_export_kms_key')
      or args.IsSpecified('periodic_export_gcs_uri')
      or args.IsSpecified('periodic_export_start_time')
  ):
    patch_request = AddFieldToUpdateMask(
        'periodic_export_config', patch_request
    )

  if args.IsSpecified('clear_periodic_export_config'):
    patch_request.instance.periodicExportConfig = None
  return patch_request


def UpdateControlledEgressConfig(unused_instance_ref, args, patch_request):
  """Hook to handle controlled egress config updates."""
  if args.IsSpecified('egress_enabled'):
    patch_request = AddFieldToUpdateMask(
        'controlled_egress_enabled', patch_request
    )
  if args.IsSpecified('marketplace_enabled'):
    if args.marketplace_enabled:
      _WarnForMarketplaceEnabledUpdate()
    patch_request = AddFieldToUpdateMask(
        'controlled_egress_config.marketplace_enabled', patch_request
    )
  if args.IsSpecified('egress_fqdns'):
    patch_request = AddFieldToUpdateMask(
        'controlled_egress_config.egress_fqdns', patch_request
    )
  return patch_request


def UpdateCatalogIntegrationOptOut(unused_instance_ref, args, patch_request):
  """Updates catalog integration opt out in the request update mask."""
  if args.IsSpecified('catalog_integration_opt_out'):
    patch_request = AddFieldToUpdateMask(
        'catalog_integration_opt_out', patch_request
    )
  return patch_request


def UpdateReleaseChannel(instance_ref, args, patch_request):
  """Hook to update release channel to the update mask of the request.

  Args:
    instance_ref: The reference to the instance being updated.
    args: The arguments passed to the update command.
    patch_request: The patch request to be sent to the API.

  Returns:
    The updated patch request.

  Raises:
    exceptions.InvalidArgumentException: If maintenance settings are specified
      when switching to the RAPID release channel.
  """
  if not args.IsSpecified('release_channel'):
    return patch_request

  api_version = instance_ref.GetCollectionInfo().api_version
  messages = apis.GetMessagesModule('looker', api_version)
  instance_msg = getattr(messages, 'Instance')
  release_channel_enum = instance_msg.ReleaseChannelValueValuesEnum

  client = apis.GetClientInstance('looker', api_version)
  service = client.projects_locations_instances

  get_request_class = getattr(
      messages, 'LookerProjectsLocationsInstancesGetRequest'
  )
  current_instance = service.Get(
      get_request_class(name=instance_ref.RelativeName())
  )

  current_channel = current_instance.releaseChannel
  current_version = current_instance.lookerVersion
  new_channel = patch_request.instance.releaseChannel

  if new_channel == release_channel_enum.RAPID:
    maintenance_fields = [
        'maintenance_window_day',
        'maintenance_window_time',
        'deny_maintenance_period_start_date',
        'deny_maintenance_period_end_date',
        'deny_maintenance_period_time',
    ]
    if any(args.IsSpecified(f) for f in maintenance_fields):
      raise exceptions.InvalidArgumentException(
          '--release-channel',
          'Maintenance window and deny maintenance periods are not supported '
          'for instances in the RAPID release channel.',
      )

  warning_msg = _GetReleaseChannelWarningMessage(
      current=current_channel,
      selected=new_channel,
      looker_version=current_version,
      release_channel_enum=release_channel_enum,
  )

  if warning_msg:
    console_io.PromptContinue(
        message=warning_msg,
        prompt_string='Do you want to proceed with the release channel switch?',
        cancel_on_no=True,
    )

  if (
      new_channel == release_channel_enum.RAPID
      and current_channel != release_channel_enum.RAPID
  ):
    patch_request.instance.maintenanceWindow = None
    patch_request.instance.denyMaintenancePeriod = None
    patch_request = AddFieldToUpdateMask('maintenance_window', patch_request)
    patch_request = AddFieldToUpdateMask(
        'deny_maintenance_period', patch_request
    )

  return AddFieldToUpdateMask('release_channel', patch_request)


def UpdateAcceleratedSecurityPatch(unused_instance_ref, args, patch_request):
  """Hook to update accelerated security patch to the update mask of the request."""
  if args.IsSpecified('accelerated_security_patch_enabled'):
    patch_request = AddFieldToUpdateMask(
        'accelerated_security_patch_enabled', patch_request
    )
  return patch_request


def _WarnForMarketplaceEnabledUpdate():
  """Adds prompt that warns about marketplace enabled update."""
  message = 'Change to instance marketplace enabled requested. '
  message += (
      'Enabling connection to the Looker Marketplace will also allow egress to'
      ' github.com.'
  )
  console_io.PromptContinue(
      message=message,
      prompt_string='Do you want to proceed with update?',
      cancel_on_no=True,
  )



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""recommender API recommendations list command."""


import itertools

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.asset import client_util
from googlecloudsdk.api_lib.recommender import locations
from googlecloudsdk.api_lib.recommender import recommendation
from googlecloudsdk.api_lib.recommender import recommenders
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.recommender import flags
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.core import log


DETAILED_HELP = {
    'EXAMPLES':
        """
          Lists recommendations for a Cloud project.
            $ {command} --project=project-id --location=global --recommender=google.compute.instance.MachineTypeRecommender
        """,
}

DISPLAY_FORMAT = """
        table(
          name.basename(): label=RECOMMENDATION_ID,
          primaryImpact.category: label=PRIMARY_IMPACT_CATEGORY,
          stateInfo.state: label=RECOMMENDATION_STATE,
          lastRefreshTime: label=LAST_REFRESH_TIME,
          priority: label=PRIORITY,
          recommenderSubtype: label=RECOMMENDER_SUBTYPE,
          description: label=DESCRIPTION
        )
    """


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class List(base.ListCommand):
  r"""List recommendations for Google Cloud resources.

  This command lists all recommendations for the specified Google Cloud
  resource, location, and recommender. If a recommender or location is not
  specified, recommendations for all supported recommenders or locations,
  respectively, are listed. If the `--recursive` flag is set,
  recommendations for child resources and projects are also listed.
  Supported recommenders can be found here:
  https://cloud.google.com/recommender/docs/recommenders.
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command.
    """
    flags.AddParentFlagsToParser(parser)
    parser.add_argument(
        '--location',
        metavar='LOCATION',
        required=False,
        help=(
            'Location to list recommendations for. If no location is specified,'
            ' recommendations for all supported locations are listed.'
            ' Not specifying a location can add 15-20 seconds to the runtime.'
        ),
    )
    parser.add_argument(
        '--recommender',
        metavar='RECOMMENDER',
        required=False,
        help=(
            'Recommender to list recommendations for. If no recommender is'
            ' specified, recommendations for all supported recommenders are'
            ' listed. Supported recommenders can be found here:'
            ' https://cloud.google.com/recommender/docs/recommenders'
            ' Not specifying a recommender can add 15-20 seconds to the'
            ' runtime.'
        ),
    )
    parser.add_argument(
        '--recursive',
        required=False,
        action=arg_parsers.StoreTrueFalseAction,
        help=("""
            In addition to listing the recommendations for the specified
            organization or folder, recursively list all of
            the recommendations for the resource's child resources, including
            their descendants (for example, a folder's sub-folders), and for the
            resource's child projects. For example, when using the
            `--recursive` flag and specifying an organization, the response
            lists all of the recommendations associated with that
            organization, all of the recommendations associated with that
            organization's folders and sub-folders, and all of the
            recommendations associated with that organization's child
            projects.  The maximum number of resources (organization,
            folders, projects, and descendant resources) that can be accessed at
            once with the `--recursive` flag is 100. For a larger
            number of nested resources, use
            [BigQuery export](https://cloud.google.com/recommender/docs/bq-export/export-recommendations-to-bq).
            Using `--recursive` can add 15-20 seconds per resource to the
            runtime.
            """),
    )
    parser.display_info.AddFormat(DISPLAY_FORMAT)

  def setArgs(self, args):
    """Setups up args to search all resources under a project, folder, or organization.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      (argparse.Namespace) args with additional parameters setup
    """

    args.read_mask = '*'
    args.asset_types = [
        # gcloud-disable-gdu-domain
        'cloudresourcemanager.googleapis.com/Project',
        # gcloud-disable-gdu-domain
        'cloudresourcemanager.googleapis.com/Folder'
    ]
    args.order_by = 'createTime'
    args.query = '*'
    if args.project:
      args.scope = 'projects/' + args.project
    if args.organization:
      args.scope = 'organizations/' + args.organization
    if args.folder:
      args.scope = 'folders/' + args.folder

    return args

  def read(self, asset_in):
    if isinstance(asset_in, list):
      return asset_in[0]
    else:
      return asset_in

  def AddResource(self, resource_location) -> bool:
    if resource_location not in self.resource_locations:
      self.resource_locations.append(resource_location)
      return True
    return False

  def searchAllResources(self, args):
    """Search all nested resources under a project, folder, or organization.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      (List): a list of all Google Cloud resource,location pairs
    """

    args = self.setArgs(args)
    client = client_util.AssetSearchClient(client_util.DEFAULT_API_VERSION)
    resources = list(client.SearchAllResources(args))
    self.resource_locations = []

    for r in resources:
      parent_resource = f'{self.read(args.scope)}/locations/{r.location}'
      if 'project' not in parent_resource:
        self.AddResource(parent_resource)

      # gcloud-disable-gdu-domain
      if r.assetType == 'cloudresourcemanager.googleapis.com/Project':
        self.AddResource(f'{self.read(r.project)}/locations/{r.location}')

      # gcloud-disable-gdu-domain
      if (
          r.assetType == 'cloudresourcemanager.googleapis.com/Folder'
          and self.AddResource(f'{self.read(r.folders)}/locations/{r.location}')
      ):
        args.scope = self.read(r.folders)
        resources.extend(client.SearchAllResources(args))
      if len(self.resource_locations) > 100:
        raise exceptions.UnsupportedOperationError(
            'The maximum number of resources (organizations, folders, projects,'
            ' and descendant resources) that can be accessed to list'
            ' recommendations is 100. To access'
            ' a larger number of resources, use BigQuery Export.'
        )
    return self.resource_locations

  def CollectAssets(self, args):
    """Iterate through search all resources response and collects unique Google Cloud resouce,location pairs.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      (List): a list of all Google Cloud resource,location pairs
    """

    # Collect Assets and Locations
    log.status.Print('Collecting Resources... This may take some time...')
    if args.recursive:
      resource_locations = self.searchAllResources(args)
    else:
      if args.location is None:
        loc_client = locations.CreateClient(self.ReleaseTrack())
        resource_locations = [
            loc.name
            for loc in loc_client.List(
                args.page_size,
                project=args.project,
                organization=args.organization,
                folder=args.folder,
                billing_account=args.billing_account,
            )
        ]
      else:
        resource_locations = [
            flags.GetResourceSegment(args) + f'/locations/{args.location}'
        ]
    return resource_locations

  def ListRecommenders(self, args):
    """List all Recommenders.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      (list) all recommenders in a list of strings
    """

    recommenders_client = recommenders.CreateClient(self.ReleaseTrack())
    recommenders_response = recommenders_client.List(args.page_size)
    return list(recommenders_response)

  def GetRecommendations(self, args, asset_recommenders):
    """Collects all recommendations for a given Google Cloud Resource.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.
      asset_recommenders: list, The list of Google Cloud resource recommender
        URL to collect recommendations

    Returns:
      (Recommendations) a iterator for all returned recommendations
    """

    recommendations = []
    recommendations_client = recommendation.CreateClient(self.ReleaseTrack())

    resource_prev = None
    location_prev = None
    for resource, location, recommender in asset_recommenders:
      if resource != resource_prev or location != location_prev:
        log.status.Print(f'Reading Recommendations for: {resource} {location}')
      resource_prev = resource
      location_prev = location
      parent_name = '/'.join([resource, location, recommender])
      new_recommendations = recommendations_client.List(
          parent_name, args.page_size
      )
      try:  # skip recommenders that the user does not have access to.
        peek = next(new_recommendations)  # execute first element of generator
      except (
          apitools_exceptions.HttpBadRequestError,
          apitools_exceptions.BadStatusCodeError,
          StopIteration,
      ):
        continue
      recommendations = itertools.chain(
          recommendations, (peek,), new_recommendations
      )

    return recommendations

  def Run(self, args):
    """Run 'gcloud recommender recommendations list'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      The list of recommendations for this project.
    """
    if args.location and args.recommender:
      recommendations_client = recommendation.CreateClient(self.ReleaseTrack())
      parent_name = flags.GetRecommenderName(args)
      return recommendations_client.List(parent_name, args.page_size)

    # Collect Assets and Locations
    resource_locations = self.CollectAssets(args)

    # collect recommendations for all recommenders
    asset_recommenders = []
    for asset in resource_locations:
      tokens = asset.split('/')
      resource = '/'.join(tokens[:2])
      location = '/'.join(tokens[2:4])
      if args.recommender is not None:
        asset_recommenders.append(
            (resource, location, f'recommenders/{args.recommender}')
        )
      else:  # loop through all recommenders
        asset_recommenders.extend([
            (resource, location, f'recommenders/{response.name}')
            for response in self.ListRecommenders(args)
        ])

    return self.GetRecommendations(args, asset_recommenders)


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA)
class ListOriginal(base.ListCommand):
  r"""List operations for a recommendation.

  This command lists all recommendations for a given Google Cloud entity ID,
  location, and recommender. Supported recommenders can be found here:
  https://cloud.google.com/recommender/docs/recommenders.
  The following Google Cloud entity types are supported: project,
  billing_account, folder and organization.
  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command.
    """
    flags.AddParentFlagsToParser(parser)
    parser.add_argument(
        '--location',
        metavar='LOCATION',
        required=True,
        help='Location to list recommendations for.',
    )
    parser.add_argument(
        '--recommender',
        metavar='RECOMMENDER',
        required=True,
        help=(
            'Recommender to list recommendations for. Supported recommenders'
            ' can be found here:'
            ' https://cloud.google.com/recommender/docs/recommenders.'
        ),
    )
    parser.display_info.AddFormat(DISPLAY_FORMAT)

  def Run(self, args):
    """Run 'gcloud recommender recommendations list'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      The list of recommendations for this project.
    """
    recommendations_client = recommendation.CreateClient(self.ReleaseTrack())
    parent_name = flags.GetRecommenderName(args)

    return recommendations_client.List(parent_name, args.page_size)



# -*- coding: utf-8 -*- # Lint as: python3
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Type definitions for Apigee Feature Templates."""

import dataclasses
import re
import typing
from typing import Any, Dict, List, Optional, TypeVar, Union

T = TypeVar("T", bound=dataclasses.dataclass)


def _to_snake_case(name: str) -> str:
  """Converts a camelCase string to snake_case."""
  # This regex matches every position that satisfies both of these conditions:
  # - not at the beginning of the string (the lookbehind `(?<!^)`)
  # - immediately followed by an uppercase letter (the lookahead `(?=[A-Z])`)
  # The matches are zero-length, so replacing them with "_" effectively just
  # means "add an underscore there".
  return re.sub(r"(?<!^)(?=[A-Z])", "_", name).lower()


def _to_camel_case(name: str) -> str:
  """Converts a snake_case string to camelCase."""
  parts = name.split("_")
  return parts[0] + "".join(x.capitalize() for x in parts[1:])


def from_dict(cls: type[T], data: dict[str, Any]) -> T:
  """Creates a dataclass instance from a dictionary."""
  field_types = {f.name: f.type for f in dataclasses.fields(cls)}
  rewritten_data = {}
  for k, v in data.items():
    # The dataclasses use snake_case to match Python conventions, but the
    # dictionary representation uses camelCase to match the YAML format.
    k = _to_snake_case(k)

    if k not in field_types:
      raise ValueError(
          f'{cls} objects do not have a field named "{k}": {data!r}'
      )
    field_type = field_types[k]
    origin_type = typing.get_origin(field_type)
    type_args = typing.get_args(field_type)

    # Make sure that nested dataclasses and lists of dataclasses are
    # converted from dicts to instances.
    if dataclasses.is_dataclass(field_type) and isinstance(v, dict):
      v = from_dict(field_type, v)
    elif origin_type and type_args:
      # Handle parameterized types: Optional, List, etc.
      # Optional[T] is represented as Union[T, None] under the hood, so checking
      # it requires matching the Union pattern instead of the Optional pattern.
      is_optional = (origin_type is Union
                     and type(None) in type_args
                     and len(type_args) == 2)

      # Determine the wrapped dataclass type, if any.
      dataclass_args = [t for t in type_args if dataclasses.is_dataclass(t)]
      inner_dataclass = dataclass_args[0] if dataclass_args else None

      if is_optional and inner_dataclass and v is not None:
        v = from_dict(inner_dataclass, v)
      elif origin_type is list and inner_dataclass:
        v = [e if dataclasses.is_dataclass(e) else from_dict(inner_dataclass, e)
             for e in v]
    rewritten_data[k] = v

  return cls(**rewritten_data)


def _snake_to_camel_dict_factory(
    data: list[tuple[str, Any]],
) -> dict[str, Any]:
  """Build a dict with camelCase keys from a list of (snake_case, value) tuples."""
  return {
      _to_camel_case(k): v
      for k, v in data
  }


def to_dict(instance: T) -> dict[str, Any]:
  """Converts a dataclass instance to a dictionary."""
  return dataclasses.asdict(instance, dict_factory=_snake_to_camel_dict_factory)


class AftSchemaError(ValueError):
  """Raised when an AFT document violates the schema contract."""


# Closed supported sets for the top-level discriminator fields. Both are sorted
# in error messages for deterministic, parity-friendly diagnostics.
SUPPORTED_GATEWAYS = frozenset({"apigee"})
SUPPORTED_SCHEMA_VERSIONS = frozenset({"1.0.0"})


def _validate_supported_field(
    field_name: str, value: str, supported_values: frozenset[str]
) -> None:
  if not value:
    raise AftSchemaError(f"missing required top-level {field_name!r} field")
  if value not in supported_values:
    supported = ", ".join(sorted(supported_values))
    raise AftSchemaError(
        f"unsupported {field_name} {value!r}; supported: {supported}"
    )


def validate_gateway(value: str) -> None:
  """Validates the top-level `gateway` field (missing-first, then unsupported).

  Args:
    value: The gateway value to validate.

  Raises:
    AftSchemaError: If `value` is empty/missing or not a supported gateway.
  """
  _validate_supported_field("gateway", value, SUPPORTED_GATEWAYS)


def validate_schema_version(value: str) -> None:
  """Validates the top-level `schemaVersion` field (missing-first, then unsupported).

  Args:
    value: The schemaVersion value to validate.

  Raises:
    AftSchemaError: If `value` is empty/missing or not a supported version.
  """
  _validate_supported_field("schemaVersion", value, SUPPORTED_SCHEMA_VERSIONS)


@dataclasses.dataclass
class Parameter:
  name: str = ""
  paths: Optional[List[str]] = None
  display_name: str = ""
  description: str = ""
  maps: Optional[Dict[str, str]] = None
  examples: List[str] = dataclasses.field(default_factory=list)
  default: str = ""


@dataclasses.dataclass
class Route:
  name: str = ""
  condition: Optional[str] = None
  target: Optional[str] = None


@dataclasses.dataclass
class Step:
  name: str = ""
  condition: Optional[str] = None


@dataclasses.dataclass
class Flow:
  name: str
  mode: Optional[str] = None
  condition: Optional[str] = None
  steps: List[Step] = dataclasses.field(default_factory=list)


@dataclasses.dataclass
class FaultRule(Flow):
  always_enforce: bool = False


@dataclasses.dataclass
class Endpoint:
  name: str = ""
  base_path: str = ""
  routes: List[Route] = dataclasses.field(default_factory=list)


@dataclasses.dataclass
class ProxyEndpoint(Endpoint):
  flows: List[Flow] = dataclasses.field(default_factory=list)
  post_client_flow: Optional[Flow] = None
  fault_rules: List[Flow] = dataclasses.field(default_factory=list)
  default_fault_rule: Optional[FaultRule] = None


@dataclasses.dataclass
class Target:
  name: str = ""
  url: str = ""
  auth: Optional[str] = None
  scopes: List[str] = dataclasses.field(default_factory=list)
  aud: Optional[str] = None


@dataclasses.dataclass
class ProxyTarget(Target):
  flows: List[Flow] = dataclasses.field(default_factory=list)
  fault_rules: List[Flow] = dataclasses.field(default_factory=list)
  default_fault_rule: Optional[FaultRule] = None
  http_target_connection: Optional[Dict[str, Any]] = None
  local_target_connection: Optional[Dict[str, Any]] = None


@dataclasses.dataclass
class Policy:
  name: str = ""
  type: str = ""
  content: Dict[str, Any] = dataclasses.field(default_factory=dict)


@dataclasses.dataclass
class Resource:
  """Represents a resource file to be included in an Apigee API proxy bundle."""

  name: str = ""
  type: str = ""
  content: str = ""


@dataclasses.dataclass
class Test:
  """Represents an Apigee API proxy test. Not yet implemented."""
  name: str = ""
  description: Optional[str] = None
  url: str = ""
  path: Optional[str] = None
  method: Optional[str] = None
  headers: List[str] = dataclasses.field(default_factory=list)
  request: Optional[str] = None
  query_params: List[str] = dataclasses.field(default_factory=list)
  variables: List[str] = dataclasses.field(default_factory=list)
  assertions: List[str] = dataclasses.field(default_factory=list)


@dataclasses.dataclass
class Template:
  """Represents an Apigee API proxy template."""

  gateway: str = ""
  schema_version: str = ""
  name: str = ""
  type: str = "template"
  priority: Optional[int] = None
  description: str = ""
  features: List[str] = dataclasses.field(default_factory=list)
  parameters: List[Parameter] = dataclasses.field(default_factory=list)
  endpoints: List[Endpoint] = dataclasses.field(default_factory=list)
  targets: List[Target] = dataclasses.field(default_factory=list)
  tests: List[Test] = dataclasses.field(default_factory=list)

  def __post_init__(self):
    validate_gateway(self.gateway)
    validate_schema_version(self.schema_version)


@dataclasses.dataclass
class Proxy:
  """Represents an Apigee API proxy."""

  gateway: str = ""
  schema_version: str = ""
  name: str = ""
  display_name: Optional[str] = None
  uid: Optional[str] = None
  type: str = "proxy"
  priority: Optional[int] = None
  categories: List[str] = dataclasses.field(default_factory=list)
  description: str = ""
  documentation: Optional[str] = None
  parameters: List[Parameter] = dataclasses.field(default_factory=list)
  endpoints: List[ProxyEndpoint] = dataclasses.field(default_factory=list)
  targets: List[ProxyTarget] = dataclasses.field(default_factory=list)
  policies: List[Policy] = dataclasses.field(default_factory=list)
  resources: List[Resource] = dataclasses.field(default_factory=list)
  tests: List[Test] = dataclasses.field(default_factory=list)

  def __post_init__(self):
    validate_gateway(self.gateway)
    validate_schema_version(self.schema_version)


@dataclasses.dataclass
class Feature:
  """Represents an abstract feature to be included in a templated Apigee proxy."""

  gateway: str = ""
  schema_version: str = ""
  name: str = ""
  display_name: Optional[str] = None
  uid: Optional[str] = None
  type: str = "feature"
  description: str = ""
  documentation: Optional[str] = None
  priority: Optional[int] = None
  categories: List[str] = dataclasses.field(default_factory=list)
  parameters: List[Parameter] = dataclasses.field(default_factory=list)
  default_endpoint: Optional[ProxyEndpoint] = None
  default_target: Optional[ProxyTarget] = None
  endpoints: List[ProxyEndpoint] = dataclasses.field(default_factory=list)
  targets: List[ProxyTarget] = dataclasses.field(default_factory=list)
  policies: List[Policy] = dataclasses.field(default_factory=list)
  resources: List[Resource] = dataclasses.field(default_factory=list)
  tests: List[Test] = dataclasses.field(default_factory=list)

  def __post_init__(self):
    validate_gateway(self.gateway)
    validate_schema_version(self.schema_version)



"""Generated message classes for cloudfunctions version v1.

Manages lightweight user-provided functions executed in response to events.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'cloudfunctions'


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class AutomaticUpdatePolicy(_messages.Message):
  r"""Security patches are applied automatically to the runtime without
  requiring the function to be redeployed.
  """



class Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. * `domain:{domain}`: The G Suite domain
      (primary) that represents all the users of that domain. For example,
      `google.com` or `example.com`. * `principal://iam.googleapis.com/locatio
      ns/global/workforcePools/{pool_id}/subject/{subject_attribute_value}`: A
      single identity in a workforce identity pool. * `principalSet://iam.goog
      leapis.com/locations/global/workforcePools/{pool_id}/group/{group_id}`:
      All workforce identities in a group. * `principalSet://iam.googleapis.co
      m/locations/global/workforcePools/{pool_id}/attribute.{attribute_name}/{
      attribute_value}`: All workforce identities with a specific attribute
      value. * `principalSet://iam.googleapis.com/locations/global/workforcePo
      ols/{pool_id}/*`: All identities in a workforce identity pool. * `princi
      pal://iam.googleapis.com/projects/{project_number}/locations/global/work
      loadIdentityPools/{pool_id}/subject/{subject_attribute_value}`: A single
      identity in a workload identity pool. * `principalSet://iam.googleapis.c
      om/projects/{project_number}/locations/global/workloadIdentityPools/{poo
      l_id}/group/{group_id}`: A workload identity pool group. * `principalSet
      ://iam.googleapis.com/projects/{project_number}/locations/global/workloa
      dIdentityPools/{pool_id}/attribute.{attribute_name}/{attribute_value}`:
      All identities in a workload identity pool with a certain attribute. * `
      principalSet://iam.googleapis.com/projects/{project_number}/locations/gl
      obal/workloadIdentityPools/{pool_id}/*`: All identities in a workload
      identity pool. * `deleted:user:{emailid}?uid={uniqueid}`: An email
      address (plus unique identifier) representing a user that has been
      recently deleted. For example,
      `alice@example.com?uid=123456789012345678901`. If the user is recovered,
      this value reverts to `user:{emailid}` and the recovered user retains
      the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `deleted:principal://iam.google
      apis.com/locations/global/workforcePools/{pool_id}/subject/{subject_attr
      ibute_value}`: Deleted single identity in a workforce identity pool. For
      example, `deleted:principal://iam.googleapis.com/locations/global/workfo
      rcePools/my-pool-id/subject/my-subject-attribute-value`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`. For an
      overview of the IAM roles and permissions, see the [IAM
      documentation](https://cloud.google.com/iam/docs/roles-overview). For a
      list of the available pre-defined roles, see
      [here](https://cloud.google.com/iam/docs/understanding-roles).
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CallFunctionRequest(_messages.Message):
  r"""Request for the `CallFunction` method.

  Fields:
    data: Required. Input to be passed to the function.
  """

  data = _messages.StringField(1)


class CallFunctionResponse(_messages.Message):
  r"""Response of `CallFunction` method.

  Fields:
    error: Either system or user-function generated error. Set if execution
      was not successful.
    executionId: Execution id of function invocation.
    result: Result populated for successful execution of synchronous function.
      Will not be populated if function does not return a result through
      context.
  """

  error = _messages.StringField(1)
  executionId = _messages.StringField(2)
  result = _messages.StringField(3)


class CloudFunction(_messages.Message):
  r"""Describes a Cloud Function that contains user computation executed in
  response to an event. It encapsulate function and triggers configurations.

  Enums:
    DockerRegistryValueValuesEnum: Docker Registry to use for this deployment.
      Deprecated: as of March 2025, `CONTAINER_REGISTRY` option is no longer
      available in response to Container Registry's deprecation:
      https://cloud.google.com/artifact-registry/docs/transition/transition-
      from-gcr Please use Artifact Registry instead, which is the default
      choice. If unspecified, it defaults to `ARTIFACT_REGISTRY`. If
      `docker_repository` field is specified, this field should either be left
      unspecified or set to `ARTIFACT_REGISTRY`.
    IngressSettingsValueValuesEnum: The ingress settings for the function,
      controlling what traffic can reach it.
    StatusValueValuesEnum: Output only. Status of the function deployment.
    VpcConnectorEgressSettingsValueValuesEnum: The egress settings for the
      connector, controlling what traffic is diverted through it.

  Messages:
    BuildEnvironmentVariablesValue: Build environment variables that shall be
      available during build time.
    EnvironmentVariablesValue: Environment variables that shall be available
      during function execution.
    LabelsValue: Labels associated with this Cloud Function.

  Fields:
    automaticUpdatePolicy: A AutomaticUpdatePolicy attribute.
    availableMemoryMb: The amount of memory in MB available for a function.
      Defaults to 256MB.
    buildDockerfile: Local path to the dockerfile for customizing the base
      image for the builder, located within the source folder. Deprecated:
      this was significantly revised before reaching public.
    buildEnvironmentVariables: Build environment variables that shall be
      available during build time.
    buildId: Output only. The Cloud Build ID of the latest successful
      deployment of the function.
    buildName: Output only. The Cloud Build Name of the function deployment.
      `projects//locations//builds/`.
    buildServiceAccount: A service account the user provides for use with
      Cloud Build. The format of this field is
      `projects/{projectId}/serviceAccounts/{serviceAccountEmail}`.
    buildWorkerPool: Name of the Cloud Build Custom Worker Pool that should be
      used to build the function. The format of this field is
      `projects/{project}/locations/{region}/workerPools/{workerPool}` where
      `{project}` and `{region}` are the project id and region respectively
      where the worker pool is defined and `{workerPool}` is the short name of
      the worker pool. If the project id is not the same as the function, then
      the Cloud Functions Service Agent (`service-@gcf-admin-
      robot.iam.gserviceaccount.com`) must be granted the role Cloud Build
      Custom Workers Builder (`roles/cloudbuild.customworkers.builder`) in the
      project.
    buildpackStack: Specifies one of the Google provided buildpack stacks.
      Deprecated: this API was suspended after private preview and was never
      revealed publicly.
    customStackUri: Deprecated: customization experience was significantly
      revised, making this field obsolete. It was never revealed to public.
      The URL of a customer provided buildpack stack.
    description: User-provided description of a function.
    dockerRegistry: Docker Registry to use for this deployment. Deprecated: as
      of March 2025, `CONTAINER_REGISTRY` option is no longer available in
      response to Container Registry's deprecation:
      https://cloud.google.com/artifact-registry/docs/transition/transition-
      from-gcr Please use Artifact Registry instead, which is the default
      choice. If unspecified, it defaults to `ARTIFACT_REGISTRY`. If
      `docker_repository` field is specified, this field should either be left
      unspecified or set to `ARTIFACT_REGISTRY`.
    dockerRepository: User-managed repository created in Artifact Registry to
      which the function's Docker image will be pushed after it is built by
      Cloud Build. May optionally be encrypted with a customer-managed
      encryption key (CMEK). If unspecified and `docker_registry` is not
      explicitly set to `CONTAINER_REGISTRY`, GCF will create and use a
      default Artifact Registry repository named 'gcf-artifacts' in the
      region. It must match the pattern
      `projects/{project}/locations/{location}/repositories/{repository}`.
      Cross-project repositories are not supported. Cross-location
      repositories are not supported. Repository format must be 'DOCKER'.
    entryPoint: The name of the function (as defined in source code) that will
      be executed. Defaults to the resource name suffix (ID of the function),
      if not specified.
    environmentVariables: Environment variables that shall be available during
      function execution.
    eventTrigger: A source that fires events in response to a condition in
      another service.
    httpsTrigger: An HTTPS endpoint type of source that can be triggered via
      URL.
    ingressSettings: The ingress settings for the function, controlling what
      traffic can reach it.
    kmsKeyName: Resource name of a KMS crypto key (managed by the user) used
      to encrypt/decrypt function resources. It must match the pattern `projec
      ts/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto
      _key}`. If specified, you must also provide an artifact registry
      repository using the `docker_repository` field that was created with the
      same KMS crypto key. The following service accounts need to be granted
      the role 'Cloud KMS CryptoKey Encrypter/Decrypter
      (roles/cloudkms.cryptoKeyEncrypterDecrypter)' on the
      Key/KeyRing/Project/Organization (least access preferred). 1. Google
      Cloud Functions service account (service-{project_number}@gcf-admin-
      robot.iam.gserviceaccount.com) - Required to protect the function's
      image. 2. Google Storage service account (service-{project_number}@gs-
      project-accounts.iam.gserviceaccount.com) - Required to protect the
      function's source code. If this service account does not exist,
      deploying a function without a KMS key or retrieving the service agent
      name provisions it. For more information, see
      https://cloud.google.com/storage/docs/projects#service-agents and
      https://cloud.google.com/storage/docs/getting-service-agent#gsutil.
      Google Cloud Functions delegates access to service agents to protect
      function resources in internal projects that are not accessible by the
      end user.
    labels: Labels associated with this Cloud Function.
    maxInstances: The limit on the maximum number of function instances that
      may coexist at a given time. In some cases, such as rapid traffic
      surges, Cloud Functions may, for a short period of time, create more
      instances than the specified max instances limit. If your function
      cannot tolerate this temporary behavior, you may want to factor in a
      safety margin and set a lower max instances value than your function can
      tolerate. See the [Max
      Instances](https://cloud.google.com/functions/docs/max-instances) Guide
      for more details.
    minInstances: A lower bound for the number function instances that may
      coexist at a given time.
    name: A user-defined name of the function. Function names must be unique
      globally and match pattern `projects/*/locations/*/functions/*`
    network: Deprecated: use vpc_connector
    onDeployUpdatePolicy: A OnDeployUpdatePolicy attribute.
    pinnedRuntimeVersionPolicy: A PinnedRuntimeVersionPolicy attribute.
    runDockerfile: Local path to the dockerfile for customizing the base image
      for the worker, located within the source folder. Deprecated: this was
      significantly revised before reaching public.
    runtime: The runtime in which to run the function. Required when deploying
      a new function, optional when updating an existing function. For a
      complete list of possible choices, see the [`gcloud` command reference](
      https://cloud.google.com/sdk/gcloud/reference/functions/deploy#--
      runtime).
    satisfiesPzi: Output only.
    satisfiesPzs: Output only.
    secretEnvironmentVariables: Secret environment variables configuration.
    secretVolumes: Secret volumes configuration.
    serviceAccountEmail: The email of the function's service account. If
      empty, defaults to `{project_id}@appspot.gserviceaccount.com`.
    sourceArchiveUrl: The Google Cloud Storage URL, starting with `gs://`,
      pointing to the zip archive which contains the function.
    sourceRepository: **Beta Feature** The source repository where a function
      is hosted.
    sourceToken: Input only. An identifier for Firebase function sources.
      Disclaimer: This field is only supported for Firebase function
      deployments.
    sourceUploadUrl: The Google Cloud Storage signed URL used for source
      uploading, generated by calling
      [google.cloud.functions.v1.GenerateUploadUrl]. The signature is
      validated on write methods (Create, Update) The signature is stripped
      from the Function object on read methods (Get, List)
    status: Output only. Status of the function deployment.
    timeout: The function execution timeout. Execution is considered failed
      and can be terminated if the function is not completed at the end of the
      timeout period. Defaults to 60 seconds.
    updateTime: Output only. The last update timestamp of a Cloud Function.
    versionId: Output only. The version identifier of the Cloud Function. Each
      deployment attempt results in a new version of a function being created.
    vpcConnector: The VPC Network Connector that this cloud function can
      connect to. It can be either the fully-qualified URI, or the short name
      of the network connector resource. The format of this field is
      `projects/*/locations/*/connectors/*` This field is mutually exclusive
      with `network` field and will eventually replace it. See [the VPC
      documentation](https://cloud.google.com/compute/docs/vpc) for more
      information on connecting Cloud projects.
    vpcConnectorEgressSettings: The egress settings for the connector,
      controlling what traffic is diverted through it.
  """

  class DockerRegistryValueValuesEnum(_messages.Enum):
    r"""Docker Registry to use for this deployment. Deprecated: as of March
    2025, `CONTAINER_REGISTRY` option is no longer available in response to
    Container Registry's deprecation: https://cloud.google.com/artifact-
    registry/docs/transition/transition-from-gcr Please use Artifact Registry
    instead, which is the default choice. If unspecified, it defaults to
    `ARTIFACT_REGISTRY`. If `docker_repository` field is specified, this field
    should either be left unspecified or set to `ARTIFACT_REGISTRY`.

    Values:
      DOCKER_REGISTRY_UNSPECIFIED: Unspecified.
      CONTAINER_REGISTRY: Docker images will be stored in multi-regional
        Container Registry repositories named `gcf`.
      ARTIFACT_REGISTRY: Docker images will be stored in regional Artifact
        Registry repositories. By default, GCF will create and use
        repositories named `gcf-artifacts` in every region in which a function
        is deployed. But the repository to use can also be specified by the
        user using the `docker_repository` field.
    """
    DOCKER_REGISTRY_UNSPECIFIED = 0
    CONTAINER_REGISTRY = 1
    ARTIFACT_REGISTRY = 2

  class IngressSettingsValueValuesEnum(_messages.Enum):
    r"""The ingress settings for the function, controlling what traffic can
    reach it.

    Values:
      INGRESS_SETTINGS_UNSPECIFIED: Unspecified.
      ALLOW_ALL: Allow HTTP traffic from public and private sources.
      ALLOW_INTERNAL_ONLY: Allow HTTP traffic from only private VPC sources.
      ALLOW_INTERNAL_AND_GCLB: Allow HTTP traffic from private VPC sources and
        through GCLB.
    """
    INGRESS_SETTINGS_UNSPECIFIED = 0
    ALLOW_ALL = 1
    ALLOW_INTERNAL_ONLY = 2
    ALLOW_INTERNAL_AND_GCLB = 3

  class StatusValueValuesEnum(_messages.Enum):
    r"""Output only. Status of the function deployment.

    Values:
      CLOUD_FUNCTION_STATUS_UNSPECIFIED: Not specified. Invalid state.
      ACTIVE: Function has been successfully deployed and is serving.
      OFFLINE: Function deployment failed and the function isn't serving.
      DEPLOY_IN_PROGRESS: Function is being created or updated.
      DELETE_IN_PROGRESS: Function is being deleted.
      UNKNOWN: Function deployment failed and the function serving state is
        undefined. The function should be updated or deleted to move it out of
        this state.
    """
    CLOUD_FUNCTION_STATUS_UNSPECIFIED = 0
    ACTIVE = 1
    OFFLINE = 2
    DEPLOY_IN_PROGRESS = 3
    DELETE_IN_PROGRESS = 4
    UNKNOWN = 5

  class VpcConnectorEgressSettingsValueValuesEnum(_messages.Enum):
    r"""The egress settings for the connector, controlling what traffic is
    diverted through it.

    Values:
      VPC_CONNECTOR_EGRESS_SETTINGS_UNSPECIFIED: Unspecified.
      PRIVATE_RANGES_ONLY: Use the VPC Access Connector only for private IP
        space from RFC1918.
      ALL_TRAFFIC: Force the use of VPC Access Connector for all egress
        traffic from the function.
    """
    VPC_CONNECTOR_EGRESS_SETTINGS_UNSPECIFIED = 0
    PRIVATE_RANGES_ONLY = 1
    ALL_TRAFFIC = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class BuildEnvironmentVariablesValue(_messages.Message):
    r"""Build environment variables that shall be available during build time.

    Messages:
      AdditionalProperty: An additional property for a
        BuildEnvironmentVariablesValue object.

    Fields:
      additionalProperties: Additional properties of type
        BuildEnvironmentVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a BuildEnvironmentVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class EnvironmentVariablesValue(_messages.Message):
    r"""Environment variables that shall be available during function
    execution.

    Messages:
      AdditionalProperty: An additional property for a
        EnvironmentVariablesValue object.

    Fields:
      additionalProperties: Additional properties of type
        EnvironmentVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a EnvironmentVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels associated with this Cloud Function.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  automaticUpdatePolicy = _messages.MessageField('AutomaticUpdatePolicy', 1)
  availableMemoryMb = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  buildDockerfile = _messages.StringField(3)
  buildEnvironmentVariables = _messages.MessageField('BuildEnvironmentVariablesValue', 4)
  buildId = _messages.StringField(5)
  buildName = _messages.StringField(6)
  buildServiceAccount = _messages.StringField(7)
  buildWorkerPool = _messages.StringField(8)
  buildpackStack = _messages.StringField(9)
  customStackUri = _messages.StringField(10)
  description = _messages.StringField(11)
  dockerRegistry = _messages.EnumField('DockerRegistryValueValuesEnum', 12)
  dockerRepository = _messages.StringField(13)
  entryPoint = _messages.StringField(14)
  environmentVariables = _messages.MessageField('EnvironmentVariablesValue', 15)
  eventTrigger = _messages.MessageField('EventTrigger', 16)
  httpsTrigger = _messages.MessageField('HttpsTrigger', 17)
  ingressSettings = _messages.EnumField('IngressSettingsValueValuesEnum', 18)
  kmsKeyName = _messages.StringField(19)
  labels = _messages.MessageField('LabelsValue', 20)
  maxInstances = _messages.IntegerField(21, variant=_messages.Variant.INT32)
  minInstances = _messages.IntegerField(22, variant=_messages.Variant.INT32)
  name = _messages.StringField(23)
  network = _messages.StringField(24)
  onDeployUpdatePolicy = _messages.MessageField('OnDeployUpdatePolicy', 25)
  pinnedRuntimeVersionPolicy = _messages.MessageField('PinnedRuntimeVersionPolicy', 26)
  runDockerfile = _messages.StringField(27)
  runtime = _messages.StringField(28)
  satisfiesPzi = _messages.BooleanField(29)
  satisfiesPzs = _messages.BooleanField(30)
  secretEnvironmentVariables = _messages.MessageField('SecretEnvVar', 31, repeated=True)
  secretVolumes = _messages.MessageField('SecretVolume', 32, repeated=True)
  serviceAccountEmail = _messages.StringField(33)
  sourceArchiveUrl = _messages.StringField(34)
  sourceRepository = _messages.MessageField('SourceRepository', 35)
  sourceToken = _messages.StringField(36)
  sourceUploadUrl = _messages.StringField(37)
  status = _messages.EnumField('StatusValueValuesEnum', 38)
  timeout = _messages.StringField(39)
  updateTime = _messages.StringField(40)
  versionId = _messages.IntegerField(41)
  vpcConnector = _messages.StringField(42)
  vpcConnectorEgressSettings = _messages.EnumField('VpcConnectorEgressSettingsValueValuesEnum', 43)


class CloudfunctionsOperationsGetRequest(_messages.Message):
  r"""A CloudfunctionsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class CloudfunctionsOperationsListRequest(_messages.Message):
  r"""A CloudfunctionsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class CloudfunctionsProjectsLocationsFunctionsCallRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsCallRequest object.

  Fields:
    callFunctionRequest: A CallFunctionRequest resource to be passed as the
      request body.
    name: Required. The name of the function to be called.
  """

  callFunctionRequest = _messages.MessageField('CallFunctionRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudfunctionsProjectsLocationsFunctionsCreateRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsCreateRequest object.

  Fields:
    cloudFunction: A CloudFunction resource to be passed as the request body.
    location: Required. The project and location in which the function should
      be created, specified in the format `projects/*/locations/*`
  """

  cloudFunction = _messages.MessageField('CloudFunction', 1)
  location = _messages.StringField(2, required=True)


class CloudfunctionsProjectsLocationsFunctionsDeleteRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsDeleteRequest object.

  Fields:
    name: Required. The name of the function which should be deleted.
  """

  name = _messages.StringField(1, required=True)


class CloudfunctionsProjectsLocationsFunctionsGenerateDownloadUrlRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsGenerateDownloadUrlRequest
  object.

  Fields:
    generateDownloadUrlRequest: A GenerateDownloadUrlRequest resource to be
      passed as the request body.
    name: The name of function for which source code Google Cloud Storage
      signed URL should be generated.
  """

  generateDownloadUrlRequest = _messages.MessageField('GenerateDownloadUrlRequest', 1)
  name = _messages.StringField(2, required=True)


class CloudfunctionsProjectsLocationsFunctionsGenerateUploadUrlRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsGenerateUploadUrlRequest
  object.

  Fields:
    generateUploadUrlRequest: A GenerateUploadUrlRequest resource to be passed
      as the request body.
    parent: The project and location in which the Google Cloud Storage signed
      URL should be generated, specified in the format
      `projects/*/locations/*`.
  """

  generateUploadUrlRequest = _messages.MessageField('GenerateUploadUrlRequest', 1)
  parent = _messages.StringField(2, required=True)


class CloudfunctionsProjectsLocationsFunctionsGetIamPolicyRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsGetIamPolicyRequest object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class CloudfunctionsProjectsLocationsFunctionsGetRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsGetRequest object.

  Fields:
    name: Required. The name of the function which details should be obtained.
    versionId: Optional. The optional version of the function whose details
      should be obtained. The version of a 1st Gen function is an integer that
      starts from 1 and gets incremented on redeployments. Each deployment
      creates a config version of the underlying function. GCF may keep
      historical configs for old versions. This field can be specified to
      fetch the historical configs. Leave it blank or set to 0 to get the
      latest version of the function.
  """

  name = _messages.StringField(1, required=True)
  versionId = _messages.IntegerField(2)


class CloudfunctionsProjectsLocationsFunctionsListRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsListRequest object.

  Fields:
    pageSize: Maximum number of functions to return per call.
    pageToken: The value returned by the last `ListFunctionsResponse`;
      indicates that this is a continuation of a prior `ListFunctions` call,
      and that the system should return the next page of data.
    parent: The project and location from which the function should be listed,
      specified in the format `projects/*/locations/*` If you want to list
      functions in all locations, use "-" in place of a location. When listing
      functions in all locations, if one or more location(s) are unreachable,
      the response will contain functions from all reachable locations along
      with the names of any unreachable locations.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudfunctionsProjectsLocationsFunctionsPatchRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsPatchRequest object.

  Fields:
    cloudFunction: A CloudFunction resource to be passed as the request body.
    name: A user-defined name of the function. Function names must be unique
      globally and match pattern `projects/*/locations/*/functions/*`
    updateMask: Required. The list of fields in `CloudFunction` that have to
      be updated.
  """

  cloudFunction = _messages.MessageField('CloudFunction', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class CloudfunctionsProjectsLocationsFunctionsSetIamPolicyRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsSetIamPolicyRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class CloudfunctionsProjectsLocationsFunctionsTestIamPermissionsRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsFunctionsTestIamPermissionsRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class CloudfunctionsProjectsLocationsListRequest(_messages.Message):
  r"""A CloudfunctionsProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class EventTrigger(_messages.Message):
  r"""Describes EventTrigger, used to request events be sent from another
  service.

  Fields:
    eventType: Required. The type of event to observe. For example:
      `providers/cloud.storage/eventTypes/object.change` and
      `providers/cloud.pubsub/eventTypes/topic.publish`. Event types match
      pattern `providers/*/eventTypes/*.*`. The pattern contains: 1.
      namespace: For example, `cloud.storage` and `google.firebase.analytics`.
      2. resource type: The type of resource on which event occurs. For
      example, the Google Cloud Storage API includes the type `object`. 3.
      action: The action that generates the event. For example, action for a
      Google Cloud Storage Object is 'change'. These parts are lower case.
    failurePolicy: Specifies policy for failed executions.
    resource: Required. The resource(s) from which to observe events, for
      example, `projects/_/buckets/myBucket`. Not all syntactically correct
      values are accepted by all services. For example: 1. The authorization
      model must support it. Google Cloud Functions only allows EventTriggers
      to be deployed that observe resources in the same project as the
      `CloudFunction`. 2. The resource type must match the pattern expected
      for an `event_type`. For example, an `EventTrigger` that has an
      `event_type` of "google.pubsub.topic.publish" should have a resource
      that matches Google Cloud Pub/Sub topics. Additionally, some services
      may support short names when creating an `EventTrigger`. These will
      always be returned in the normalized "long" format. See each *service's*
      documentation for supported formats.
    service: The hostname of the service that should be observed. If no string
      is provided, the default service implementing the API will be used. For
      example, `storage.googleapis.com` is the default for all event types in
      the `google.storage` namespace.
  """

  eventType = _messages.StringField(1)
  failurePolicy = _messages.MessageField('FailurePolicy', 2)
  resource = _messages.StringField(3)
  service = _messages.StringField(4)


class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class FailurePolicy(_messages.Message):
  r"""Describes the policy in case of function's execution failure. If empty,
  then defaults to ignoring failures (i.e. not retrying them).

  Fields:
    retry: If specified, then the function will be retried in case of a
      failure.
  """

  retry = _messages.MessageField('Retry', 1)


class GenerateDownloadUrlRequest(_messages.Message):
  r"""Request of `GenerateDownloadUrl` method.

  Fields:
    versionId: The optional version of function. If not set, default, current
      version is used.
  """

  versionId = _messages.IntegerField(1, variant=_messages.Variant.UINT64)


class GenerateDownloadUrlResponse(_messages.Message):
  r"""Response of `GenerateDownloadUrl` method.

  Fields:
    downloadUrl: The generated Google Cloud Storage signed URL that should be
      used for function source code download.
  """

  downloadUrl = _messages.StringField(1)


class GenerateUploadUrlRequest(_messages.Message):
  r"""Request of `GenerateSourceUploadUrl` method.

  Fields:
    kmsKeyName: Resource name of a KMS crypto key (managed by the user) used
      to encrypt/decrypt function source code objects in intermediate Cloud
      Storage buckets. When you generate an upload url and upload your source
      code, it gets copied to an intermediate Cloud Storage bucket. The source
      code is then copied to a versioned directory in the sources bucket in
      the consumer project during the function deployment. It must match the
      pattern `projects/{project}/locations/{location}/keyRings/{key_ring}/cry
      ptoKeys/{crypto_key}`. The Google Cloud Functions service account
      (service-{project_number}@gcf-admin-robot.iam.gserviceaccount.com) must
      be granted the role 'Cloud KMS CryptoKey Encrypter/Decrypter
      (roles/cloudkms.cryptoKeyEncrypterDecrypter)' on the
      Key/KeyRing/Project/Organization (least access preferred). GCF will
      delegate access to the Google Storage service account in the internal
      project.
  """

  kmsKeyName = _messages.StringField(1)


class GenerateUploadUrlResponse(_messages.Message):
  r"""Response of `GenerateSourceUploadUrl` method.

  Fields:
    uploadUrl: The generated Google Cloud Storage signed URL that should be
      used for a function source code upload. The uploaded file should be a
      zip archive which contains a function.
  """

  uploadUrl = _messages.StringField(1)


class HttpsTrigger(_messages.Message):
  r"""Describes HttpsTrigger, could be used to connect web hooks to function.

  Enums:
    SecurityLevelValueValuesEnum: The security level for the function.

  Fields:
    securityLevel: The security level for the function.
    url: Output only. The deployed url for the function.
  """

  class SecurityLevelValueValuesEnum(_messages.Enum):
    r"""The security level for the function.

    Values:
      SECURITY_LEVEL_UNSPECIFIED: Unspecified.
      SECURE_ALWAYS: Requests for a URL that match this handler that do not
        use HTTPS are automatically redirected to the HTTPS URL with the same
        path. Query parameters are reserved for the redirect.
      SECURE_OPTIONAL: Both HTTP and HTTPS requests with URLs that match the
        handler succeed without redirects. The application can examine the
        request to determine which protocol was used and respond accordingly.
    """
    SECURITY_LEVEL_UNSPECIFIED = 0
    SECURE_ALWAYS = 1
    SECURE_OPTIONAL = 2

  securityLevel = _messages.EnumField('SecurityLevelValueValuesEnum', 1)
  url = _messages.StringField(2)


class ListFunctionsResponse(_messages.Message):
  r"""Response for the `ListFunctions` method.

  Fields:
    functions: The functions that match the request.
    nextPageToken: If not empty, indicates that there may be more functions
      that match the request; this value should be passed in a new
      google.cloud.functions.v1.ListFunctionsRequest to get more functions.
    unreachable: Locations that could not be reached. The response does not
      include any functions from these locations.
  """

  functions = _messages.MessageField('CloudFunction', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class OnDeployUpdatePolicy(_messages.Message):
  r"""Security patches are only applied when a function is redeployed.

  Fields:
    runtimeVersion: Output only. Contains the runtime version which was used
      during latest function deployment.
  """

  runtimeVersion = _messages.StringField(1)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadataV1(_messages.Message):
  r"""Metadata describing an Operation

  Enums:
    TypeValueValuesEnum: Type of operation.

  Messages:
    RequestValue: The original request that started the operation.

  Fields:
    buildId: The Cloud Build ID of the function created or updated by an API
      call. This field is only populated for Create and Update operations.
    buildName: The Cloud Build Name of the function deployment. This field is
      only populated for Create and Update operations.
      `projects//locations//builds/`.
    request: The original request that started the operation.
    sourceToken: An identifier for Firebase function sources. Disclaimer: This
      field is only supported for Firebase function deployments.
    target: Target of the operation - for example
      `projects/project-1/locations/region-1/functions/function-1`
    type: Type of operation.
    updateTime: The last update timestamp of the operation.
    versionId: Version id of the function created or updated by an API call.
      This field is only populated for Create and Update operations.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Type of operation.

    Values:
      OPERATION_UNSPECIFIED: Unknown operation type.
      CREATE_FUNCTION: Triggered by CreateFunction call
      UPDATE_FUNCTION: Triggered by UpdateFunction call
      DELETE_FUNCTION: Triggered by DeleteFunction call.
    """
    OPERATION_UNSPECIFIED = 0
    CREATE_FUNCTION = 1
    UPDATE_FUNCTION = 2
    DELETE_FUNCTION = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class RequestValue(_messages.Message):
    r"""The original request that started the operation.

    Messages:
      AdditionalProperty: An additional property for a RequestValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a RequestValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  buildId = _messages.StringField(1)
  buildName = _messages.StringField(2)
  request = _messages.MessageField('RequestValue', 3)
  sourceToken = _messages.StringField(4)
  target = _messages.StringField(5)
  type = _messages.EnumField('TypeValueValuesEnum', 6)
  updateTime = _messages.StringField(7)
  versionId = _messages.IntegerField(8)


class PinnedRuntimeVersionPolicy(_messages.Message):
  r"""The function is pinned to a specific runtime version and it will not
  receive security patches, even after redeploying.

  Fields:
    runtimeVersion: The runtime version this function is pinned to. This
      version will be used every time this function is deployed.
  """

  runtimeVersion = _messages.StringField(1)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** ``` { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: -
  members: - user:mike@example.com - group:admins@example.com -
  domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com
  role: roles/resourcemanager.organizationAdmin - members: -
  user:eve@example.com role: roles/resourcemanager.organizationViewer
  condition: title: expirable access description: Does not grant access after
  Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
  etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features,
  see the [IAM documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class Retry(_messages.Message):
  r"""Describes the retry policy in case of function's execution failure. A
  function execution will be retried on any failure. A failed execution will
  be retried up to 7 days with an exponential backoff (capped at 10 seconds).
  Retried execution is charged as any other execution.
  """



class SecretEnvVar(_messages.Message):
  r"""Configuration for a secret environment variable. It has the information
  necessary to fetch the secret value from secret manager and expose it as an
  environment variable.

  Fields:
    key: Name of the environment variable.
    projectId: Project identifier (preferably project number but can also be
      the project ID) of the project that contains the secret. If not set, it
      will be populated with the function's project assuming that the secret
      exists in the same project as of the function.
    secret: Name of the secret in secret manager (not the full resource name).
    version: Version of the secret (version number or the string 'latest'). It
      is recommended to use a numeric version for secret environment variables
      as any updates to the secret value is not reflected until new instances
      start.
  """

  key = _messages.StringField(1)
  projectId = _messages.StringField(2)
  secret = _messages.StringField(3)
  version = _messages.StringField(4)


class SecretVersion(_messages.Message):
  r"""Configuration for a single version.

  Fields:
    path: Relative path of the file under the mount path where the secret
      value for this version will be fetched and made available. For example,
      setting the mount_path as '/etc/secrets' and path as `/secret_foo` would
      mount the secret value file at `/etc/secrets/secret_foo`.
    version: Version of the secret (version number or the string 'latest'). It
      is preferable to use `latest` version with secret volumes as secret
      value changes are reflected immediately.
  """

  path = _messages.StringField(1)
  version = _messages.StringField(2)


class SecretVolume(_messages.Message):
  r"""Configuration for a secret volume. It has the information necessary to
  fetch the secret value from secret manager and make it available as files
  mounted at the requested paths within the application container. Secret
  value is not a part of the configuration. Every filesystem read operation
  performs a lookup in secret manager to retrieve the secret value.

  Fields:
    mountPath: The path within the container to mount the secret volume. For
      example, setting the mount_path as `/etc/secrets` would mount the secret
      value files under the `/etc/secrets` directory. This directory will also
      be completely shadowed and unavailable to mount any other secrets.
      Recommended mount paths: /etc/secrets Restricted mount paths: /cloudsql,
      /dev/log, /pod, /proc, /var/log
    projectId: Project identifier (preferrably project number but can also be
      the project ID) of the project that contains the secret. If not set, it
      will be populated with the function's project assuming that the secret
      exists in the same project as of the function.
    secret: Name of the secret in secret manager (not the full resource name).
    versions: List of secret versions to mount for this secret. If empty, the
      `latest` version of the secret will be made available in a file named
      after the secret under the mount point.
  """

  mountPath = _messages.StringField(1)
  projectId = _messages.StringField(2)
  secret = _messages.StringField(3)
  versions = _messages.MessageField('SecretVersion', 4, repeated=True)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Google Cloud services (such as Projects) might
      reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: `paths: "bindings, etag"`
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class SourceRepository(_messages.Message):
  r"""Describes SourceRepository, used to represent parameters related to
  source repository where a function is hosted.

  Fields:
    deployedUrl: Output only. The URL pointing to the hosted repository where
      the function were defined at the time of deployment. It always points to
      a specific commit in the format described above.
    url: The URL pointing to the hosted repository where the function is
      defined. There are supported Cloud Source Repository URLs in the
      following formats: To refer to a specific commit: `https://source.develo
      pers.google.com/projects/*/repos/*/revisions/*/paths/*` To refer to a
      moveable alias (branch):
      `https://source.developers.google.com/projects/*/repos/*/moveable-
      aliases/*/paths/*` In particular, to refer to HEAD use `master` moveable
      alias. To refer to a specific fixed alias (tag):
      `https://source.developers.google.com/projects/*/repos/*/fixed-
      aliases/*/paths/*` You may omit `paths/*` if you want to use the main
      directory. The function response may add an empty `/paths/` to the URL.
  """

  deployedUrl = _messages.StringField(1)
  url = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as `*` or `storage.*`) are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
encoding.AddCustomJsonFieldMapping(
    CloudfunctionsProjectsLocationsFunctionsGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from cloudsdk.google.protobuf import struct_pb2  # type: ignore


__protobuf__ = proto.module(
    package='google.cloud.aiplatform.v1',
    manifest={
        'Type',
        'Schema',
    },
)


class Type(proto.Enum):
    r"""Type contains the list of OpenAPI data types as defined by
    https://swagger.io/docs/specification/data-models/data-types/

    Values:
        TYPE_UNSPECIFIED (0):
            Not specified, should not be used.
        STRING (1):
            OpenAPI string type
        NUMBER (2):
            OpenAPI number type
        INTEGER (3):
            OpenAPI integer type
        BOOLEAN (4):
            OpenAPI boolean type
        ARRAY (5):
            OpenAPI array type
        OBJECT (6):
            OpenAPI object type
        NULL (7):
            Null type
    """
    TYPE_UNSPECIFIED = 0
    STRING = 1
    NUMBER = 2
    INTEGER = 3
    BOOLEAN = 4
    ARRAY = 5
    OBJECT = 6
    NULL = 7


class Schema(proto.Message):
    r"""Defines the schema of input and output data. This is a subset of the
    `OpenAPI 3.0 Schema
    Object <https://spec.openapis.org/oas/v3.0.3#schema-object>`__.

    Attributes:
        type_ (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.Type):
            Optional. Data type of the schema field.
        format_ (str):
            Optional. The format of the data. For ``NUMBER`` type,
            format can be ``float`` or ``double``. For ``INTEGER`` type,
            format can be ``int32`` or ``int64``. For ``STRING`` type,
            format can be ``email``, ``byte``, ``date``, ``date-time``,
            ``password``, and other formats to further refine the data
            type.
        title (str):
            Optional. Title for the schema.
        description (str):
            Optional. Describes the data. The model uses
            this field to understand the purpose of the
            schema and how to use it. It is a best practice
            to provide a clear and descriptive explanation
            for the schema and its properties here, rather
            than in the prompt.
        nullable (bool):
            Optional. Indicates if the value of this
            field can be null.
        default (google.protobuf.struct_pb2.Value):
            Optional. Default value to use if the field
            is not specified.
        items (googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.Schema):
            Optional. If type is ``ARRAY``, ``items`` specifies the
            schema of elements in the array.
        min_items (int):
            Optional. If type is ``ARRAY``, ``min_items`` specifies the
            minimum number of items in an array.
        max_items (int):
            Optional. If type is ``ARRAY``, ``max_items`` specifies the
            maximum number of items in an array.
        enum (MutableSequence[str]):
            Optional. Possible values of the field. This field can be
            used to restrict a value to a fixed set of values. To mark a
            field as an enum, set ``format`` to ``enum`` and provide the
            list of possible values in ``enum``. For example:

            1. To define directions:
               ``{type:STRING, format:enum, enum:["EAST", "NORTH", "SOUTH", "WEST"]}``
            2. To define apartment numbers:
               ``{type:INTEGER, format:enum, enum:["101", "201", "301"]}``
        properties (MutableMapping[str, googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.Schema]):
            Optional. If type is ``OBJECT``, ``properties`` is a map of
            property names to schema definitions for each property of
            the object.
        property_ordering (MutableSequence[str]):
            Optional. Order of properties displayed or
            used where order matters. This is not a standard
            field in OpenAPI specification, but can be used
            to control the order of properties.
        required (MutableSequence[str]):
            Optional. If type is ``OBJECT``, ``required`` lists the
            names of properties that must be present.
        min_properties (int):
            Optional. If type is ``OBJECT``, ``min_properties``
            specifies the minimum number of properties that can be
            provided.
        max_properties (int):
            Optional. If type is ``OBJECT``, ``max_properties``
            specifies the maximum number of properties that can be
            provided.
        minimum (float):
            Optional. If type is ``INTEGER`` or ``NUMBER``, ``minimum``
            specifies the minimum allowed value.
        maximum (float):
            Optional. If type is ``INTEGER`` or ``NUMBER``, ``maximum``
            specifies the maximum allowed value.
        min_length (int):
            Optional. If type is ``STRING``, ``min_length`` specifies
            the minimum length of the string.
        max_length (int):
            Optional. If type is ``STRING``, ``max_length`` specifies
            the maximum length of the string.
        pattern (str):
            Optional. If type is ``STRING``, ``pattern`` specifies a
            regular expression that the string must match.
        example (google.protobuf.struct_pb2.Value):
            Optional. Example of an instance of this
            schema.
        any_of (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.Schema]):
            Optional. The instance must be valid against any (one or
            more) of the subschemas listed in ``any_of``.
        additional_properties (google.protobuf.struct_pb2.Value):
            Optional. If ``type`` is ``OBJECT``, specifies how to handle
            properties not defined in ``properties``. If it is a boolean
            ``false``, no additional properties are allowed. If it is a
            schema, additional properties are allowed if they conform to
            the schema.
        ref (str):
            Optional. Allows referencing another schema definition to
            use in place of this schema. The value must be a valid
            reference to a schema in ``defs``.

            For example, the following schema defines a reference to a
            schema node named "Pet":

            type: object properties: pet: ref: #/defs/Pet defs: Pet:
            type: object properties: name: type: string

            The value of the "pet" property is a reference to the schema
            node named "Pet". See details in
            https://json-schema.org/understanding-json-schema/structuring
        defs (MutableMapping[str, googlecloudsdk.generated_clients.gapic_clients.aiplatform_v1.types.Schema]):
            Optional. ``defs`` provides a map of schema definitions that
            can be reused by ``ref`` elsewhere in the schema. Only
            allowed at root level of the schema.
    """

    type_: 'Type' = proto.Field(
        proto.ENUM,
        number=1,
        enum='Type',
    )
    format_: str = proto.Field(
        proto.STRING,
        number=7,
    )
    title: str = proto.Field(
        proto.STRING,
        number=24,
    )
    description: str = proto.Field(
        proto.STRING,
        number=8,
    )
    nullable: bool = proto.Field(
        proto.BOOL,
        number=6,
    )
    default: struct_pb2.Value = proto.Field(
        proto.MESSAGE,
        number=23,
        message=struct_pb2.Value,
    )
    items: 'Schema' = proto.Field(
        proto.MESSAGE,
        number=2,
        message='Schema',
    )
    min_items: int = proto.Field(
        proto.INT64,
        number=21,
    )
    max_items: int = proto.Field(
        proto.INT64,
        number=22,
    )
    enum: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=9,
    )
    properties: MutableMapping[str, 'Schema'] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=3,
        message='Schema',
    )
    property_ordering: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=25,
    )
    required: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )
    min_properties: int = proto.Field(
        proto.INT64,
        number=14,
    )
    max_properties: int = proto.Field(
        proto.INT64,
        number=15,
    )
    minimum: float = proto.Field(
        proto.DOUBLE,
        number=16,
    )
    maximum: float = proto.Field(
        proto.DOUBLE,
        number=17,
    )
    min_length: int = proto.Field(
        proto.INT64,
        number=18,
    )
    max_length: int = proto.Field(
        proto.INT64,
        number=19,
    )
    pattern: str = proto.Field(
        proto.STRING,
        number=20,
    )
    example: struct_pb2.Value = proto.Field(
        proto.MESSAGE,
        number=4,
        message=struct_pb2.Value,
    )
    any_of: MutableSequence['Schema'] = proto.RepeatedField(
        proto.MESSAGE,
        number=11,
        message='Schema',
    )
    additional_properties: struct_pb2.Value = proto.Field(
        proto.MESSAGE,
        number=26,
        message=struct_pb2.Value,
    )
    ref: str = proto.Field(
        proto.STRING,
        number=27,
    )
    defs: MutableMapping[str, 'Schema'] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=28,
        message='Schema',
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create command to create a new resource of Custom Mirroring profile."""

from googlecloudsdk.api_lib.network_security.security_profiles import mirroring_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_security import sp_flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log

DETAILED_HELP = {
    'DESCRIPTION': """

          Create a new Custom Mirroring Security Profile.

        """,
    'EXAMPLES': """
          To create a Custom Mirroring Security Profile named `mirroring-profile` linked to a Mirroring Endpoint Group (q.v.), run:

              $ {command} mirroring-profile --description="A Mirroring Profile" \
                --mirroring-endpoint-group=projects/my-project/locations/global/mirroringEndpointGroups/my-mep

        """,
}

_BROKER_RELEASE_TRACKS = (base.ReleaseTrack.ALPHA,)
_PROJECT_SCOPE_SUPPORTED_TRACKS = (
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
)


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Create(base.CreateCommand):
  """Create a new Custom Mirroring Profile."""

  detailed_help = DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    project_scope_supported = (
        cls.ReleaseTrack() in _PROJECT_SCOPE_SUPPORTED_TRACKS
    )
    sp_flags.AddSecurityProfileResource(
        parser, cls.ReleaseTrack(), project_scope_supported
    )
    sp_flags.AddProfileDescription(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, False)
    labels_util.AddCreateLabelsFlags(parser)
    sp_flags.AddMirroringEndpointGroupResource(
        cls.ReleaseTrack(), parser, project_scope_supported
    )
    if cls.ReleaseTrack() in _BROKER_RELEASE_TRACKS:
      sp_flags.AddCustomMirroringDeploymentGroupsArg(parser)

  def Run(self, args):
    result = args.CONCEPTS.security_profile.Parse()
    security_profile = result.result

    project_scoped = (
        result.concept_type.name
        == sp_flags.PROJECT_SECURITY_PROFILE_RESOURCE_COLLECTION
    )
    client = mirroring_api.Client(self.ReleaseTrack(), project_scoped)

    description = args.description
    labels = labels_util.ParseCreateArgs(
        args, client.messages.SecurityProfile.LabelsValue
    )
    is_async = args.async_
    mirroring_endpoint_group = args.CONCEPTS.mirroring_endpoint_group.Parse()

    mirroring_deployment_groups = None
    if self.ReleaseTrack() in _BROKER_RELEASE_TRACKS:
      mirroring_deployment_groups = args.mirroring_deployment_groups

    response = client.CreateCustomMirroringProfile(
        sp_id=security_profile.Name(),
        parent=security_profile.Parent().RelativeName(),
        description=description,
        labels=labels,
        mirroring_endpoint_group=mirroring_endpoint_group.RelativeName(),
        mirroring_deployment_groups=mirroring_deployment_groups,
    )

    # Return the in-progress operation if async is requested.
    if is_async:
      operation_id = response.name
      log.status.Print(
          'Check for operation completion status using operation ID:',
          operation_id,
      )
      return response

    # Default operation poller if async is not specified.
    return client.WaitForOperation(
        operation_ref=client.GetOperationsRef(response),
        message='Waiting for security-profile [{}] to be created'.format(
            security_profile.RelativeName()
        ),
        has_result=True,
    )
